#!/bin/bash
#
# Copyright 2006 Ludwig Zeller
#
# Usage: timeslots.sh [-o outname] [-0 length of index] [-a first frame] [-b last frame] [-f frame step] [-w image width]  [-s slice height] [ -p phi degree rotation ] [-m move slit] [-y offset] [-z end of offset]  filebody -m moving: -y (scroll start) and -z (scroll end) have to be set in this mode.
#
##################################################
# This file is part of timeslots.
# timeslots is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
# timeslots is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
##################################################

#-------------------------------- default options
	EXTENSION=bmp		
	OUTNAME=out					# o:
	OUT_EXT=.jpg					# extension of outfile, defines the compression for imagemagick
	TEMP_FILE=/tmp/tmp`date +%N`.rgb			# create random file for tmp

	FIELDWIDTH=5					# fill the zeros 0:
	FRAME_START=1					# a:
	FRAME_STEP=1					# f:

	IMAGE_W=1440   					# Image width w:
	IMAGE_Y=1080   					# Image width w:
	OFFSET_START=0					# y offset of rotated image! y:
	OFFSET=0
	SLICE_HEIGHT=8					# s:
	REVERSE=0						#reverse processing =false
	VERBOSE=0
	MOVE=0						# move scanline, like scrolling in bob okanes script -> overwrites static offset parameter!
	PHI=0						#rotate PHI degrees


#-------------------------------- fetch and overwrite options
	while getopts  "p:vri:o:0:a:b:f:w:h:y:s:mz:e:" o
	do		
		case "$o" in
			o)	OUTNAME="$OPTARG";;
			e)	OUT_EXT="$OPTARG";;
			0)	FIELDWIDTH="$OPTARG";;
			a)	FRAME_START="$OPTARG";;
			b)	FRAME_END="$OPTARG";;
			f)	FRAME_STEP="$OPTARG";;
			w)	IMAGE_W="$OPTARG";;
			h)	IMAGE_H="$OPTARG";;
			y)	OFFSET_START="$OPTARG";OFFSET="$OPTARG";;
			z)	OFFSET_END="$OPTARG";;
			s)	SLICE_HEIGHT="$OPTARG";;
			v)	VERBOSE=1;;
			r)	REVERSE=1;;
			p)	PHI="$OPTARG";;
			m)	MOVE=1;;
			
			[?])	echo >&2 "Usage: timeslots.sh [-o outname] [-0 length of index] [-a first frame] [-b last frame] [-f frame step] [-w image width]  [-s slice height] [ -p phi degree rotation ] [-m move slit] [-y offset] [-z end of offset]  filebody
			
-m moving: -y (scroll start) and -z (scroll end) have to be set in this mode."
				exit 1;;
		esac
	done
	shift $(($OPTIND-1))				# this is to point to the actual argument FILEBODY -> $1
	FILEBODY=$1	

#-------------------------------- reasoning
	# missing frame end
	if [ -z $FRAME_END ]; then
		echo "You need to specify the number of the last frame: -b number"
		exit 1;
	fi

	#missing filebody	
	if [ -z $FILEBODY ]; then
		echo "Error: FILEBODY argument is missing.";
		exit 1;
	fi

	# end > start
	if [ $FRAME_START -ge $FRAME_END ]; then
		echo "Error: First frame has to be lower than last frame.";
		exit 1;
	fi
	# step > 0	
	if [ $FRAME_STEP -lt 1 ]; then
		echo "Error: $FRAME_STEP < 1";
		exit 1;
	fi	

#-------------------------------- dynamic params
	BS=$(($IMAGE_W * 3))				# the bytes of a single line	
	TOTAL=$(((($FRAME_END-$FRAME_START))/$FRAME_STEP))	# the number of all frames
	
	OFFSET=$(($BS*$OFFSET))					# skip bytes of lines
	OFFSET=$(($OFFSET+54))					# skip bmp header, which is always 54 bytes.
		
#-------------------------------- remove tmp, otherwise new data would be appended
	if [ -e $TEMP_FILE ]; then
		rm $TEMP_FILE
	fi
	touch $TEMP_FILE
	
#-------------------------------- statistics

	# finished frames = iterations = frame_step
	
	# get time 
	BEGIN_SEC=`date +%s`


#-------------------------------- main loop

	# main loop function

	function main () 
	{
		if [ $MOVE -eq 1 ]; then				# handle moving
			OFFSET=`echo "scale=6; x=$CURRENT / $FRAME_END * ( $OFFSET_END - $SLICE_HEIGHT - $OFFSET_START ) + $OFFSET_START; scale=0; x / 1" | bc -l`
		fi
		
		FILENAME=`printf "%s%0${FIELDWIDTH}d.%s" $FILEBODY $CURRENT $EXTENSION`
		if [ -e $FILENAME ]; then
		
			
			# the optimised cat, prints only the necessary bytes0			
		       	#dd if=$FILENAME bs=$BS count=$SLICE_HEIGHT skip=$OFFSET >> $TEMP_FILE	2> /dev/null
# 		       	dd if=$FILENAME bs=$BS count=$SLICE_HEIGHT skip="b $OFFSET" >> $TEMP_FILE #2> /dev/null
			ddim $FILENAME $OFFSET $(($SLICE_HEIGHT*$BS)) >> $TEMP_FILE 2> /dev/null

			if [ $VERBOSE -eq 1 ]; then
 				echo -e "Currentfile: `basename $FILENAME`; Progress: $(((((($CURRENT-$FRAME_START))*100))/$TOTAL))%\r\c" >&2 

			fi
		else 
			echo "Error: File $FILENAME is missing."
			exit 1;
		fi
	}

	# handle  reversing
	if [ $REVERSE -eq 0 ]; then					# normal mode
	
		CURRENT=$FRAME_START
		while [ $CURRENT -le $FRAME_END ]; do			# $FRAME_START <= $FRAME_END
			main;	
			CURRENT=$(($CURRENT + $FRAME_STEP));
		done;
	else 	
		# reverse mode
		CURRENT=$FRAME_END
		while [ $CURRENT -ge $FRAME_START ]; do			# $FRAME_START >= $FRAME_END
			main;
			CURRENT=$(($CURRENT - $FRAME_STEP));
		done;			
	fi

#-------------------------------- statistics
	END_SEC=`date +%s`
	if [ $VERBOSE -eq 1 ]; then	
		echo ""
		DURATION=$(($END_SEC-$BEGIN_SEC))
		if [ $DURATION -gt 0 ]; then
			echo "Process took $DURATION sec, that's $(($TOTAL/$DURATION)) FPS";
		else
			echo "Statistics cannot be calculated because of rounding issues.";
		fi		
	fi

#-------------------------------- save jpg out of tmp.rgb
	OUT_HEIGHT=$(($SLICE_HEIGHT*(($TOTAL+1))))				# the height of the output file after all iterations
	echo $SLICE_HEIGHT
	if [ $PHI -ne 0 ]; then						# !=
		ROTATE="-rotate $PHI";
	else
		ROTATE="";
	fi
	
	# create outname
	if [ $MOVE -eq 1 ]; then
		MOV_TEXT="-mov-"
	fi
	if [ $REVERSE -eq 1 ]; then
		REV_TEXT="-rev-"
	fi
	
	OUTNAME=$OUTNAME$MOV_TEXT$REV_TEXT$FRAME_STEP-$SLICE_HEIGHT-$FRAME_START-$FRAME_END-$OFFSET_END-$OFFSET_START$OUT_EXT	
	convert -size ${IMAGE_W}x${OUT_HEIGHT} -depth 8 $ROTATE $TEMP_FILE $OUTNAME
	if [ $VERBOSE -eq 1 ]; then
		echo >&2 "Saved to $OUTNAME"
	fi
	rm $TEMP_FILE
	
	exit 0
